@@ -19,7 +19,7 @@ use warnings;
 use Carp;
 use HiPi::Utils qw( is_raspberry );
 
-our $VERSION ='0.45';
+our $VERSION ='0.46';
 
 our $sudoprog = 'sudo';
 
@@ -33,24 +33,27 @@ our $_safepath = '/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin';
 
 sub use_sudo {
     if(@_) {
-        $_cansudostash{usesudo} = $_[0];
+        %_cansudostash = (
+            lasteuid => -1,
+            cansudo  => undef,
+            usesudo  => $_[0],
+        );
     }
     return $_cansudostash{usesudo};
 }
 
 sub can_sudo {
-    return 0 unless $_cansudostash{usesudo};
-    return 0 unless is_raspberry;
-    # return stashed result if we have it
+    # back quickly if we don't need to check user
+    return 0 unless ( $_cansudostash{usesudo} && is_raspberry && $< );
+    
+    # have we checked for the current user    
     if( $_cansudostash{lasteuid} == $< ) {
         return $_cansudostash{cansudo};
     }
     
     $_cansudostash{lasteuid} = $<;
     
-    if( $_cansudostash{lasteuid} == 0 ) {
-        $_cansudostash{cansudo} = 0;
-    } else {
+    {
         local $ENV{PATH} = $_safepath;
         $_cansudostash{cansudo} = ( system('sudo -V >/dev/null 2>&1') ) ? 0 : 1;
     }
@@ -62,7 +65,7 @@ sub system_sudo {
     my $command = shift;
     return 0 unless is_raspberry;
     local $ENV{PATH} = $_safepath;
-    if( $< && can_sudo() ) {
+    if( can_sudo ) {
         $command = qq($sudoprog $command);
     }
     system($command);
@@ -72,7 +75,7 @@ sub qx_sudo {
     my $command = shift;
     return '' unless is_raspberry;
     local $ENV{PATH} = $_safepath;
-    if( $< && can_sudo() ) {
+    if( can_sudo ) {
         $command = qq($sudoprog $command);
     }
     qx($command);
@@ -82,7 +85,7 @@ sub system_sudo_shell {
     my $command = shift;
     return 0 unless is_raspberry;
     local $ENV{PATH} = $_safepath;
-    if( $< && can_sudo() ) {
+    if( can_sudo ) {
         $command = qq($sudoprog sh -c '$command');
     }
     system($command);
@@ -92,7 +95,7 @@ sub qx_sudo_shell {
     my $command = shift;
     return '' unless is_raspberry;
     local $ENV{PATH} = $_safepath;
-    if( $< && can_sudo ) {
+    if( can_sudo ) {
         $command = qq($sudoprog sh -c '$command');
     }
     qx($command);
