@@ -2,7 +2,7 @@
 # Package       HiPi::Constant
 # Description:  Utility constants for HiPi
 # Created       Fri Nov 23 22:23:29 2012
-# SVN Id        $Id: Constant.pm 13 2015-03-29 21:36:38Z Mark Dootson $
+# SVN Id        $Id: Constant.pm 84 2016-04-03 02:08:01Z Mark Dootson $
 # Copyright:    Copyright (c) 2012 Mark Dootson
 # Licence:      This work is free software; you can redistribute it and/or modify it 
 #               under the terms of the GNU General Public License as published by the 
@@ -56,6 +56,25 @@ $EXPORT_TAGS{raspberry} = \@_rpi_const;
 $EXPORT_TAGS{i2c} = \@_i2c_const;
 $EXPORT_TAGS{wiring} = \@_wiring_const;
 
+#-------------------------------------------
+#  I2C read mode
+#-------------------------------------------
+
+use constant {
+    I2C_READMODE_SYSTEM => 0,
+    I2C_READMODE_REPEATED_START => 1,
+    I2C_READMODE_START_STOP => 2,  
+};
+
+{
+    my @const = qw(
+        I2C_READMODE_SYSTEM I2C_READMODE_REPEATED_START I2C_READMODE_START_STOP  
+    );
+
+    push @EXPORT_OK, @const;
+    $EXPORT_TAGS{i2c_readmode}  = \@const;
+}
+
 #-------------------------------------------
 # Pin mode constants
 #-------------------------------------------
