@@ -2,7 +2,7 @@
 # Package       HiPi::Device::I2C
 # Description:  Wrapper for I2C communucation
 # Created       Fri Nov 23 13:55:49 2012
-# SVN Id        $Id: I2C.pm 78 2016-03-29 13:53:19Z Mark Dootson $
+# SVN Id        $Id: I2C.pm 71 2016-03-15 16:22:29Z Mark Dootson $
 # Copyright:    Copyright (c) 2012 Mark Dootson
 # Licence:      This work is free software; you can redistribute it and/or modify it 
 #               under the terms of the GNU General Public License as published by the 
@@ -23,13 +23,13 @@ use Fcntl;
 use XSLoader;
 use Carp;
 use Time::HiRes qw( usleep );
-use HiPi::Constant qw( :raspberry );
+use HiPi::Constant qw( :raspberry :i2c_readmode);
 use HiPi::Utils qw( is_raspberry );
 use Try::Tiny;
 
-our $VERSION ='0.45';
+our $VERSION ='0.46';
 
-__PACKAGE__->create_accessors( qw ( fh fno address i2cbuffer busmode ) );
+__PACKAGE__->create_accessors( qw ( fh fno address i2cbuffer busmode readmode ) );
 
 XSLoader::load('HiPi::Device::I2C', $VERSION) if is_raspberry;
 
@@ -88,12 +88,15 @@ _register_exported_constants qw(
     
 );
 
-
 our @_moduleinfo = (
-    { name => 'i2c_bcm2708', params => { baudrate => 100000 }, },
+    { name => 'i2c_bcm2708', params => {}, },
     { name => 'i2c_dev',     params => {}, },
 );
 
+my $_module_param_defaults = {
+    i2c_bcm2708 => { baudrate => 100000, combined => 'N' },
+    i2c_dev => {},
+};
 
 sub get_module_info {
     return @_moduleinfo;
@@ -101,12 +104,14 @@ sub get_module_info {
 
 {
     my $discard = __PACKAGE__->get_baudrate();
+    $discard = __PACKAGE__->get_combined();
 }
 
 sub get_baudrate {
-    my $class = shift;
+    my ($class) = @_;
     my $baudrate = HiPi::qx_sudo_shell('/bin/cat /sys/module/i2c_bcm2708/parameters/baudrate 2>&1');
     if($?) {
+        $_moduleinfo[0]->{params}->{baudrate} = $_module_param_defaults->{i2c_bcm2708}->{baudrate};
         return $_moduleinfo[0]->{params}->{baudrate};
     }
     chomp($baudrate);
@@ -116,13 +121,40 @@ sub get_baudrate {
 
 sub set_baudrate {
     my ($class, $newrate) = @_;
-    croak('Usage HiPi::Device::I2C->set_baudrate( $baudrate )') if ( defined($newrate) && ref($newrate) );
     $newrate ||= 3816;
     $newrate = 3816 if $newrate < 3816;
     $_moduleinfo[0]->{params}->{baudrate} = $newrate;
+    
+    # baudrate requires module reload
+    
     $class->load_modules(1);
 }
 
+sub get_combined {
+    my ($class, $force) = @_;
+    $force ||= 0;
+    if (defined($_moduleinfo[0]->{params}->{combined}) && !$force ) {
+        return $_moduleinfo[0]->{params}->{combined};
+    }
+    my $combined = HiPi::qx_sudo_shell('/bin/cat /sys/module/i2c_bcm2708/parameters/combined 2>&1');
+    if($?) {
+         $_moduleinfo[0]->{params}->{combined} = $_module_param_defaults->{i2c_bcm2708}->{combined};
+        return $_moduleinfo[0]->{params}->{combined};
+    }
+    chomp($combined);
+    $_moduleinfo[0]->{params}->{combined} = $combined;
+    return $combined;
+}
+
+sub set_combined {
+    my ($class, $newval) = @_;
+    $newval //= 'N';
+    $newval = uc($newval);
+    croak('Usage HiPi::Device::I2C->set_combined( "Y|N" )') unless ( $newval =~ /^Y|N$/ );
+    $_moduleinfo[0]->{params}->{combined} = $newval;
+    HiPi::qx_sudo_shell(qq(/bin/echo $newval > /sys/module/i2c_bcm2708/parameters/combined));
+}
+
 sub get_device_list {
     # get the devicelist
     opendir my $dh, '/dev' or croak qq(Failed to open dev : $!);
@@ -145,12 +177,18 @@ sub new {
         fno          => undef,
         i2cbuffer    => [],
         busmode      => 'smbus',
+        readmode     => I2C_READMODE_SYSTEM,
     );
     
     foreach my $key (sort keys(%userparams)) {
         $params{$key} = $userparams{$key};
     }
-
+    
+    # if we are going to be writing module params we need permissions
+    if ( ($params{readmode} == I2C_READMODE_REPEATED_START) || ($params{readmode} == I2C_READMODE_START_STOP ) ) {
+        HiPi::use_sudo(1);
+    }
+    
     my $fh = IO::File->new( $params{devicename}, O_RDWR, 0 ) or croak qq(open error on $params{devicename}: $!\n);
     
     $params{fh}  = $fh;
@@ -221,11 +259,34 @@ sub bus_write_error {
 
 sub bus_read {
     my ($self, $cmdval, $numbytes) = @_;
-    if( $self->busmode eq 'smbus' ) {
-        return $self->smbus_read( $cmdval, $numbytes );
-    } else {
-        return $self->i2c_read_register($cmdval, $numbytes );
+
+    # check if we need to change read mode
+    my $resetcombined = undef;
+    if ($self->readmode == I2C_READMODE_START_STOP  ) {
+        my $combined = $self->get_combined;
+        if ( $combined ne 'N') {
+            $resetcombined = $combined;
+            $self->set_combined('N');
+        }
+    } elsif($self->readmode == I2C_READMODE_REPEATED_START  ) {
+        my $combined = $self->get_combined;
+        if ( $combined ne 'Y') {
+            $resetcombined = $combined;
+            $self->set_combined('Y');
+        }
     }
+    
+    my @arrayreturn  = ();
+        
+    if( $self->busmode eq 'smbus' ) {       
+        @arrayreturn = $self->smbus_read( $cmdval, $numbytes );
+    } else {
+        @arrayreturn = $self->i2c_read_register($cmdval, $numbytes );
+    }
+    
+    $self->set_combined($resetcombined) if $resetcombined;
+    
+    return @arrayreturn;
 }
 
 sub bus_read_bits {
@@ -262,6 +323,20 @@ sub bus_write_bits {
     $self->bus_write($register, @bytes);
 }
 
+#-------------------------------------------
+# HiPi::BCM2835 IC2 compatibility
+#-------------------------------------------
+
+sub delay {
+    my($class, $millis) = @_;
+    usleep( $millis * 1000);
+}
+
+sub delayMicroseconds {
+    my($class, $micros) = @_;
+    usleep($micros);
+}
+
 #-------------------------------------------
 # I2C interface
 #-------------------------------------------
