@@ -17,12 +17,12 @@ package HiPi::Interface::MPL3115A2;
 use strict;
 use warnings;
 use parent qw( HiPi::Interface );
-use HiPi::Constant qw( :raspberry );
-use HiPi::BCM2835::I2C qw( :i2c );
+use HiPi::Constant qw( :raspberry :i2c_readmode );
+use Carp;
 
 our $VERSION = '0.26';
 
-__PACKAGE__->create_accessors( qw( address peripheral osdelay ) );
+__PACKAGE__->create_accessors( qw( address peripheral osdelay backend readmode ) );
 
 our @EXPORT = ();
 our @EXPORT_OK = ();
@@ -227,6 +227,9 @@ use constant {
     
     MPL_OVERSAMPLE_MASK  => 0b00111000,
     
+    MPL_BB_I2C_PERI_0    => 0x10,
+    MPL_BB_I2C_PERI_1    => 0x20,
+    
 };
 
 {
@@ -380,12 +383,14 @@ use constant {
 sub new {
     my ($class, %userparams) = @_;
     my %params = (
-        peripheral  => ( RPI_BOARD_REVISION == 1 ) ? BB_I2C_PERI_0 : BB_I2C_PERI_1,
+        peripheral  => ( RPI_BOARD_REVISION == 1 ) ? MPL_BB_I2C_PERI_0 : MPL_BB_I2C_PERI_1,
+        devicename  => ( RPI_BOARD_REVISION == 1 ) ? '/dev/i2c-0' : '/dev/i2c-1',
         address     => 0x60,
         device      => undef,
         osdelay     => MPL_OSREAD_DELAY,
-        
+        backend     => 'i2c',
         _function_mode => 'hipi',
+        readmode    => I2C_READMODE_REPEATED_START,
     );
     
     # get user params
@@ -394,15 +399,40 @@ sub new {
     }
     
     unless( defined($params{device}) ) {
-        require HiPi::BCM2835::I2C;
-        $params{device} = HiPi::BCM2835::I2C->new(
-            peripheral  => $params{peripheral},
-            address     => $params{address},
-            _function_mode  => $params{_function_mode},
-        );
+        if ( $params{backend} && $params{backend} =~ /^i2c|smbus$/) {
+            require HiPi::Device::I2C;
+            $params{device} = HiPi::Device::I2C->new(
+                devicename  => $params{devicename},
+                address     => $params{address},
+                busmode     => $params{backend},
+                readmode    => $params{readmode},
+            );
+        } elsif ( $params{backend} && $params{backend} eq 'bcm2835') {
+            require HiPi::BCM2835::I2C;
+            $params{device} = HiPi::BCM2835::I2C->new(
+                peripheral  => $params{peripheral},
+                address     => $params{address},
+                readmode    => $params{readmode},
+                _function_mode  => $params{_function_mode},
+            );
+        } else {
+            croak q(invalid backend specified);
+        }
     }
     
     my $self = $class->SUPER::new(%params);
+    # init
+    {
+        my $maxloop = 0;
+        while ( $maxloop++ < 20 ) {
+            $self->sysmod;
+            last if( $self->who_am_i && $self->who_am_i == 0xC4);
+            $self->device->delay(100);
+        }
+        $self->device->delay(100);
+        $self->sysmod;
+    }
+    
     return $self;
 }
 
@@ -493,21 +523,21 @@ sub pack_pressure {
 
 sub sysmod {
     my $self = shift;
-    ( $self->device->i2c_read_register_rs(MPL_REG_SYSMOD, 1))[0];
+    ( $self->device->bus_read(MPL_REG_SYSMOD, 1))[0];
 }
 
 sub who_am_i {
     my $self = shift;
-    ( $self->device->i2c_read_register_rs(MPL_REG_WHO_AM_I, 1))[0];
+    ( $self->device->bus_read(MPL_REG_WHO_AM_I, 1))[0];
 }
 
 sub active {
     my ($self, $set) = @_;
-    my ( $curreg ) = $self->device->i2c_read_register_rs(MPL_REG_CTRL_REG1, 1);
+    my ( $curreg ) = $self->device->bus_read(MPL_REG_CTRL_REG1, 1);
     my $rval = $curreg & MPL_CTRL_REG1_SBYB;
     if (defined($set)) {
         my $setmask = ( $set ) ? MPL_CTRL_REG1_SBYB | $curreg : $curreg &~MPL_CTRL_REG1_SBYB;
-        $self->device->i2c_write(MPL_REG_CTRL_REG1, $setmask);
+        $self->device->bus_write(MPL_REG_CTRL_REG1, $setmask);
         $rval = $setmask & MPL_CTRL_REG1_SBYB;
     }
     return $rval;
@@ -515,14 +545,14 @@ sub active {
 
 sub reboot {
     my $self = shift;
-    $self->device->i2c_write_error(MPL_REG_CTRL_REG1, MPL_CTRL_REG1_RST);
+    $self->device->bus_write_error(MPL_REG_CTRL_REG1, MPL_CTRL_REG1_RST);
     $self->device->delay(100);
 }
 
 
 sub oversample {
     my($self, $newval) = @_;
-    my ( $curreg ) = $self->device->i2c_read_register_rs(MPL_REG_CTRL_REG1, 1);
+    my ( $curreg ) = $self->device->bus_read(MPL_REG_CTRL_REG1, 1);
     my $currentval = $curreg & MPL_OVERSAMPLE_MASK;
     if(defined($newval)) {
         $newval &= MPL_OVERSAMPLE_MASK;
@@ -530,8 +560,8 @@ sub oversample {
             if( $curreg & MPL_CTRL_REG1_SBYB ) {
                 croak('cannot set oversample rate while system is active');
             }
-            $self->device->i2c_write(MPL_REG_CTRL_REG1, $curreg | $newval );
-            ( $curreg ) = $self->device->i2c_read_register_rs(MPL_REG_CTRL_REG1, 1);
+            $self->device->bus_write(MPL_REG_CTRL_REG1, $curreg | $newval );
+            ( $curreg ) = $self->device->bus_read(MPL_REG_CTRL_REG1, 1);
             $currentval = $curreg & MPL_OVERSAMPLE_MASK;
         }
     }
@@ -569,7 +599,7 @@ sub raw {
 
 sub mode {
     my($self, $newmode) = @_;
-    my ( $curreg ) = $self->device->i2c_read_register_rs(MPL_REG_CTRL_REG1, 1);
+    my ( $curreg ) = $self->device->bus_read(MPL_REG_CTRL_REG1, 1);
     my $currentmode = ( $curreg & MPL_CTRL_REG1_ALT ) ? MPL_FUNC_ALTITUDE : MPL_FUNC_PRESSURE;
     if(defined($newmode)) {
         unless( $currentmode == $newmode ) {
@@ -577,8 +607,8 @@ sub mode {
                 croak('cannot set altitude / pressure mode while system is active');
             }
             my $setmask = ($newmode == MPL_FUNC_ALTITUDE) ? $curreg | MPL_CTRL_REG1_ALT : $curreg &~MPL_CTRL_REG1_ALT;
-            $self->device->i2c_write(MPL_REG_CTRL_REG1, $setmask );
-            ( $curreg ) = $self->device->i2c_read_register_rs(MPL_REG_CTRL_REG1, 1);
+            $self->device->bus_write(MPL_REG_CTRL_REG1, $setmask );
+            ( $curreg ) = $self->device->bus_read(MPL_REG_CTRL_REG1, 1);
             $currentmode = ( $curreg & MPL_CTRL_REG1_ALT ) ? MPL_FUNC_ALTITUDE : MPL_FUNC_PRESSURE;
         }
     }
@@ -605,7 +635,7 @@ sub os_altitude {
 
 sub os_any_data {
     my $self = shift;
-    my ( $curreg ) = $self->device->i2c_read_register_rs(MPL_REG_CTRL_REG1, 1);
+    my ( $curreg ) = $self->device->bus_read(MPL_REG_CTRL_REG1, 1);
     
     my $currentmode = ( $curreg & MPL_CTRL_REG1_ALT ) ? MPL_FUNC_ALTITUDE : MPL_FUNC_PRESSURE;
     my $oversample  = ( $curreg & MPL_OVERSAMPLE_MASK );
@@ -617,16 +647,16 @@ sub os_any_data {
     my $delayms = $self->delay_from_oversample($oversample);
         
     # clear any one shot bit
-    $self->device->i2c_write(MPL_REG_CTRL_REG1, $curreg &~MPL_CTRL_REG1_OST );
+    $self->device->bus_write(MPL_REG_CTRL_REG1, $curreg &~MPL_CTRL_REG1_OST );
     # set one shot bit
-    $self->device->i2c_write(MPL_REG_CTRL_REG1, $curreg | MPL_CTRL_REG1_OST );
+    $self->device->bus_write(MPL_REG_CTRL_REG1, $curreg | MPL_CTRL_REG1_OST );
     
     # wait before read
     $self->device->delay($delayms);
         
     # read data       
     my( $pmsb, $pcsb, $plsb, $tmsb, $tlsb)
-        = $self->device->i2c_read_register_rs(MPL_REG_OUT_P_MSB, 5);
+        = $self->device->bus_read(MPL_REG_OUT_P_MSB, 5);
     
     # convert pressure / altitude data
     my $pdata;
@@ -640,7 +670,7 @@ sub os_any_data {
     my $tdata = $self->unpack_temperature( $tmsb, $tlsb );
     
     # restore REG1 clearing any one shot bit
-    $self->device->i2c_write(MPL_REG_CTRL_REG1, $restorereg );
+    $self->device->bus_write(MPL_REG_CTRL_REG1, $restorereg );
     
     # return both
     return ( $pdata, $tdata );    
@@ -650,7 +680,7 @@ sub os_both_data {
     my($self, $function) = @_;
     $function //= MPL_FUNC_PRESSURE; # default it not defined
     
-    my ( $curreg ) = $self->device->i2c_read_register_rs(MPL_REG_CTRL_REG1, 1);
+    my ( $curreg ) = $self->device->bus_read(MPL_REG_CTRL_REG1, 1);
     
     my $currentmode   = ( $curreg & MPL_CTRL_REG1_ALT ) ? MPL_FUNC_ALTITUDE : MPL_FUNC_PRESSURE;
     my $currentactive = $curreg & 0x01;
@@ -664,7 +694,7 @@ sub os_both_data {
         ? $curreg | MPL_CTRL_REG1_ALT
         : $curreg &~MPL_CTRL_REG1_ALT;
     
-    $self->device->i2c_write(MPL_REG_CTRL_REG1, $ctrlmask );
+    $self->device->bus_write(MPL_REG_CTRL_REG1, $ctrlmask );
     $self->os_any_data;
 }
 
