@@ -2,7 +2,7 @@
 # Package       HiPi::Interface::SerLCD
 # Description:  SerLCD RX Enabled LCD Controller
 # Created       Sat Nov 24 20:48:42 2012
-# SVN Id        $Id: SerLCD.pm 1026 2013-03-11 08:55:02Z Mark Dootson $
+# SVN Id        $Id: SerLCD.pm 22 2016-03-04 21:38:30Z Mark Dootson $
 # Copyright:    Copyright (c) 2012 Mark Dootson
 # Licence:      This work is free software; you can redistribute it and/or modify it 
 #               under the terms of the GNU General Public License as published by the 
@@ -114,28 +114,21 @@ sub update_baudrate {
     my $self = shift;
     my $baud = $self->device->baudrate;
     my $bflag;
-    given( $baud ) {
-        when( [ 2400 ] ) {
-            $bflag = 11;
-        }
-        when( [ 4800 ] ) {
-            $bflag = 12;
-        }
-        when( [ 9600 ] ) {
-            $bflag = 13;
-        }
-        when( [ 14400 ] ) {
-            $bflag = 14;
-        }
-        when( [ 19200 ] ) {
-            $bflag = 15;
-        }
-        when( [ 38400 ] ) {
-            $bflag = 16;
-        }
-        default {
-            croak(qq(The baudrate of the serial device is not supported by the LCD controller));
-        }
+    
+    if ($baud == 2400) {
+        $bflag = 11;
+    } elsif ($baud == 4800) {
+        $bflag = 12;
+    } elsif ($baud == 9600) {
+        $bflag = 13;
+    } elsif ($baud == 14400) {
+        $bflag = 14;
+    } elsif ($baud == 19200) {
+        $bflag = 15;
+    } elsif ($baud == 38400) {
+        $bflag = 16;
+    } else {
+        croak(qq(The baudrate of the serial device is not supported by the LCD controller));
     }
     
     $self->send_special_command( $bflag );
