@@ -2,7 +2,7 @@
 # Package       HiPi::Interrupt::Handler
 # Description:  Main Thread Interrupt Handler
 # Created       Wed Apr 24 16:22:56 2013
-# SVN Id        $Id: Handler.pm 1760 2013-05-21 02:24:24Z Mark Dootson $
+# SVN Id        $Id: Handler.pm 22 2016-03-04 21:38:30Z Mark Dootson $
 # Copyright:    Copyright (c) 2013 Mark Dootson
 # Licence:      This work is free software; you can redistribute it and/or modify it 
 #               under the terms of the GNU General Public License as published by the 
@@ -60,31 +60,25 @@ sub register_callback {
     if( ref($codref) ne 'CODE' ) {
         croak('code reference parameter is not a code ref');
     }
-    given( lc($cbname) ) {
-        when(['start']) {
-            $self->cbackstart($codref);
-        }
-        when(['add']) {
-            $self->cbackadd($codref);
-        }
-        when(['remove']) {
-            $self->cbackremove($codref);
-        }
-        when(['interrupt']) {
-            $self->cbackinterrupt($codref);
-        }
-        when(['stop']) {
-            $self->cbackstop($codref);
-        }
-        when(['continue']) {
-            $self->cbackcontinue($codref);
-        }
-        when(['error']) {
-            $self->cbackerror($codref);
-        }
-        default {
-            croak(qq(unknown callback name $cbname));
-        }
+    
+    my $checkval = lc($cbname);
+    
+    if ( $checkval eq 'start') {
+        $self->cbackstart($codref);
+    } elsif( $checkval eq 'add') {
+        $self->cbackadd($codref);
+    } elsif( $checkval eq 'remove') {
+        $self->cbackremove($codref);
+    } elsif( $checkval eq 'interrupt') {
+        $self->cbackinterrupt($codref);
+    } elsif( $checkval eq 'stop') {
+        $self->cbackstop($codref);
+    }  elsif( $checkval eq 'continue') {
+        $self->cbackcontinue($codref);
+    } elsif( $checkval eq 'error') {
+        $self->cbackerror($codref);
+    } else {
+        croak(qq(unknown callback name $cbname));
     }
 }
 
@@ -114,25 +108,22 @@ sub poll {
         while (defined(my $msg = HiPi::Interrupt->tqueue->dequeue_nb)) {
             $actions ++;
             my $mobj = HiPi::Interrupt::Message->new($msg);
-            given( $mobj->action ) {
-                when( ['add'] ) {
-                    $self->on_action_add($mobj);
-                }
-                when( ['remove'] ) {
-                    $self->on_action_remove($mobj);
-                }
-                when( ['interrupt'] ) {
-                    $self->on_action_interrupt($mobj);
-                }
-                when( ['stop'] ) {
-                    # we ignore stop messages that
-                    # should currently only come
-                    # when application is closing
-                    $actions --;
-                }
-                default {
-                    $self->on_action_error($mobj);
-                }
+            
+            my $checkval = $mobj->action || 'undefined';
+            
+            if( $checkval eq 'add') {
+                $self->on_action_add($mobj);
+            } elsif( $checkval eq 'remove') {
+                $self->on_action_remove($mobj);
+            } elsif( $checkval eq 'interrupt') {
+                $self->on_action_interrupt($mobj);
+            } elsif( $checkval eq 'stop') {
+                # we ignore stop messages that
+                # should currently only come
+                # when application is closing
+                $actions --;
+            } else {
+                $self->on_action_error($mobj);
             }
         }
         $self->sleep_timeout($self->timeout) if !$actions;
@@ -161,19 +152,15 @@ sub add_pin {
         $pinclass = $self->pinref->{$pin}
     }
     
-    given( lc($pinclass) ) {
-        when( ['gpio'] ) {
-            $self->add_gpio_pin($pin);
-        }
-        when( ['bcmd'] ) {
-            $self->add_bcmd_pin($pin);
-        }
-        when( ['wire'] ) {
-            $self->add_wire_pin($pin);
-        }
-        default {
-            croak(qq(unknown pin class $pinclass));
-        }
+    my $checkval = lc($pinclass);
+    if ( $checkval eq 'gpio') {
+        $self->add_gpio_pin($pin);
+    } elsif($checkval eq 'bcmd') {
+        $self->add_bcmd_pin($pin);
+    } elsif($checkval eq 'wire') {
+        $self->add_wire_pin($pin);
+    } else {
+        croak(qq(unknown pin class $pinclass));
     }
 }
 
@@ -197,19 +184,15 @@ sub remove_pin {
         $pinclass = $self->pinref->{$pin}
     }
     
-    given( lc($pinclass) ) {
-        when( ['gpio'] ) {
-            $self->remove_gpio_pin($pin);
-        }
-        when( ['bcmd'] ) {
-            $self->remove_bcmd_pin($pin);
-        }
-        when( ['wire'] ) {
-            $self->remove_wire_pin($pin);
-        }
-        default {
-            croak(qq(unknown pin class $pinclass));
-        }
+    my $checkval = lc($pinclass);
+    if ( $checkval eq 'gpio') {
+        $self->remove_gpio_pin($pin);
+    } elsif($checkval eq 'bcmd') {
+        $self->remove_bcmd_pin($pin);
+    } elsif($checkval eq 'wire') {
+        $self->remove_wire_pin($pin);
+    } else {
+        croak(qq(unknown pin class $pinclass));
     }
 }
 
